/*

Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands

*/

import java.io.*;

/**
 * Deamon which copy lines from an InputStream to a Reporter.
 * That is, upon creation a thread is started which sits in a loop
 * reading lines from the given InputStream and writing them
 * as log messages to the given Reporter.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class Logger implements Runnable {

 private DataInput in;
 private PrintStream out;
 private String prefix;
 private Reporter reporter;

 /**
  * Create a log forwarding thread.
  * The prefix <code>name+": "</code> will be appended to every
  * line.
  */
 public Logger(InputStream in,Reporter reporter,String name)
 throws IOException
 {
	this.in=new DataInputStream(in);
	this.reporter=reporter;
	this.prefix=name+": ";
	new Thread(this).start();
 }

 public void run() {
		while(true) {
		try {
			String line=in.readLine();
			if(line==null) return;
			reporter.Log(prefix + line);
		} catch (EOFException e) {
			return;
		} catch (Exception e) {
			reporter.Fatal("Exception in "+prefix + e);
			return;
		}
	}

 }

}
