/*
 
Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands

This file was adapted from an example by Testing Technologies.
 
*/
 
import de.tu_berlin.cs.uebb.muttcn.runtime.*;
import org.etsi.ttcn.tci.*;
import org.etsi.ttcn.tri.*;
import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;
import org.etsi.ttcn.tci.TciTypeClass;
import com.testingtech.ttcn.tri.TriMessageImpl;
import java.io.*;


/**
 * TTCN-3 codec for simulated time and simple mCRL specification.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class MyCodec extends AbstractBaseCodec implements TciCDProvided
{

    private TciCDRequired typeServer;
    private Type charstringType;
    private Reporter reporter;

    public MyCodec(RB rb,Reporter reporter)
    {
        super(rb);
	this.reporter=reporter;
        typeServer = RB.getTciCDRequired();
        charstringType = typeServer.getCharstring();
    }


    public Value decode(TriMessage message, Type type) 
    {
        Value returnVal = null;

	//reporter.Log("decoding "+message+" as "+type+" ("+type.getName()+")");

	if (type.equals(charstringType)){
		CharstringValue retVal=(CharstringValue)type.newInstance();
		retVal.setString(new String(message.getEncodedMessage()));
		return retVal;
	}
	if(type.getName().equals("TermMsg")) try {
		DataInputStream src=new DataInputStream(new ByteArrayInputStream(message.getEncodedMessage()));
		RecordValue retVal=(RecordValue)type.newInstance();
		IntegerValue tmp;
		tmp=((IntegerValue)retVal.getField("tag"));
		int tag=src.readByte();
		tmp.setInt(tag);
		retVal.setField("tag",tmp);
		if(tag==1){
			((IntegerValue)retVal.getField("count")).setInt(src.readInt());
		} else {
			((IntegerValue)retVal.getField("count")).setInt(0);
		}
		String fields[]=retVal.getFieldNames();
		//for(int i = 0;i<fields.length;i++){
		//	reporter.Log("field "+fields[i]+"="+retVal.getField(fields[i]));
		//}
		//reporter.Log("returning "+retVal);
		return retVal;
	} catch (IOException e) {
		RB.getTciTMProvided().tciError("Exception "+e);
		return null;
	}

	RB.getTciTMProvided().tciError("Unsupported Type " + type + ". Decoding failed.");
        return returnVal;
    }

	public TriMessage encode(Value value)
	{
	 	//reporter.Log("encoding "+value+" ("+value.getType().getName()+")");
		String type=value.getType().getName();
		if (type.equals("charstring")) {
			return new TriMessageImpl(((CharstringValue)value).getString().getBytes());
		}
		if (type.equals("TermMsg")) try {
			ByteArrayOutputStream data=new ByteArrayOutputStream();
			DataOutputStream dst=new DataOutputStream(data);
			int tag=((IntegerValue)((RecordValue)value).getField("tag")).getInt();
			dst.writeByte((byte)tag);
			if (tag==1) dst.writeInt(((IntegerValue)((RecordValue)value).getField("count")).getInt());
			dst.flush();
			return new TriMessageImpl(data.toByteArray());
		} catch (IOException e) {
			RB.getTciTMProvided().tciError("Exception "+e);
			return null;
		}
		RB.getTciTMProvided().tciError("encoding of "+value+" failed");
		return null;
	}
	
}

