/*
 
Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands
 
*/
 
import java.io.*;
import java.util.*;
import aterm.*;
import aterm.pure.*;

/**
 * Wrapper for the mCRL stepper.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class Stepper {

  private Process stepper;
  private int init[]=new int[2];
  private DataOutputStream request;
  private DataInputStream result;
  private String label[]=new String[1024];
  private int labelcount=0;
  private int currentlabel=-1;
  private int currentstate[]=new int[2];
  private PureFactory atf=new PureFactory();
  private Reporter reporter;

  public Stepper(String tbf,Reporter reporter){
	try {
		this.reporter=reporter;
		stepper=Runtime.getRuntime().exec("stepper "+tbf);
		new Logger(stepper.getErrorStream(),reporter, "StepperProxy");
		request=new DataOutputStream(stepper.getOutputStream());
		result=new DataInputStream(stepper.getInputStream());
		init[0]=result.readInt();
		init[1]=result.readInt();
		reporter.Log("stepper started.");
	} catch (IOException e) {
	}
  }

  public int[] getInit(){
	return init;
  }

  public int[] getState(){
	return currentstate;
  }

  public int getIntLabel(){
	return currentlabel;
  }

  public String getLabel(){
	return label[currentlabel];
  }

  public String getLabel(int lbl){
	return label[lbl];
  }

  public int getLabelCount(){
	return labelcount;
  }

  public void explore(int[] state){
	try {
		request.writeInt(state[0]);
		request.writeInt(state[1]);
		request.flush();
	} catch (IOException e) {
	}
  }

  public boolean next() {
	try {
		currentlabel=result.readInt();
		if(currentlabel==-1) return false;
		if(currentlabel==labelcount){
			label[labelcount]=result.readUTF();
			ATerm t=atf.parse(label[labelcount]);
			labelcount++;
		}
		currentstate[0]=result.readInt();
		currentstate[1]=result.readInt();
		return true;
	} catch (IOException e) {
		System.exit(1);
	}
	return false;
  }

  public void stop(){
	try {
		request.close();
	} catch (IOException e) {
	}
  }

}
