/*
 
Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands
 
*/


/**
 * Idleness handler.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class TermHandler implements TermHandlerComm {

  private TermHandlerComm NextHandler;
  private boolean dirty=true;
  private boolean idle=false;
  private TermMsg buffer=null;
  private int count=0;
  private Reporter reporter;
  private String prefix;

  public TermHandler(Reporter r,String who,TermHandlerComm next){
	prefix="Termhandler("+who+"): ";
	NextHandler=next;
	reporter=r;
  }

  public synchronized void TermMsgSend(TermMsg msg){
	if (idle) {
		buffer=null;
		if (msg.tag==TermMsg.IDLE) {
			//reporter.Log(prefix+"got IDLE");
			if (dirty) {
				msg.tag=TermMsg.RUNNING;
				dirty=false;
			}
			msg.count+=count;
			NextHandler.TermMsgSend(msg);
			return;
		}
		if (msg.tag==TermMsg.RUNNING) {
			//reporter.Log(prefix+"got RUNNING");
			dirty=false;
			msg.count+=count;
			NextHandler.TermMsgSend(msg);
			return;
		}
		if (msg.tag==TermMsg.TERM) {
			//reporter.Log(prefix+"got TERM");
			if (dirty) reporter.Fatal(prefix+"dirty on termination!");
			if (!idle) reporter.Fatal(prefix+"non-idle on termination!");
			dirty=true;
			count=0;
			idle=true;
			NextHandler.TermMsgSend(msg);
			return;		
		}
		reporter.Fatal(prefix+"protocol error");
	} else {
		//reporter.Log(prefix+"still busy");
		buffer=msg;
		return;
	}
  }

  public synchronized void Send(){
	count++;
	dirty=true;
  }

  public synchronized void Idle(){
	//reporter.Log(prefix+"entering idle state");
	idle=true;
	if (buffer!=null) {
		TermMsgSend(buffer);
	}
  }

  public synchronized void Activate(){
	//reporter.Log(prefix+"leaving idle state");
	idle=false;
	dirty=true;
  }

  public synchronized void Recv(){
	count--;
	dirty=true;
	idle=false;
  }

}
