/*
 
Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands
 
*/


import java.util.*;

/**
 * Time manager.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class TimeManager implements TermHandlerComm {

  private HashSet clients=new HashSet();
  private Reporter reporter;
  private TermHandlerComm NextHandler=this;

  public TimeManager(Reporter r){
	reporter=r;
	r.Log("Time manager created.");
  }

  public synchronized TermHandlerComm RegisterHandler(TermHandlerComm h){
	TermHandlerComm tmp=NextHandler;
	NextHandler=h;
	return tmp;
  }

  public synchronized TermHandler getTermHandler(String who){
	TermHandler h=new TermHandler(reporter,who,NextHandler);
	NextHandler=h;
	return h;
  }

  public synchronized void StartProgression(){
	TermMsg msg=new TermMsg();
	msg.tag=TermMsg.IDLE;
	msg.count=0;
	NextHandler.TermMsgSend(msg);
	reporter.Log("Time manager started.");
  }

  public synchronized void add(TimedUnit u){
	clients.add(u);
  }

  public synchronized void TermMsgSend(TermMsg msg){
	//reporter.Log("dealing with message");
	if (msg.tag==TermMsg.IDLE && msg.count==0) {
		msg.tag=TermMsg.TERM;
	} else {
		if (msg.tag==TermMsg.TERM) {
			reporter.Log("Time manager: advancing time");
			Iterator i=clients.iterator();
			while(i.hasNext()){
				((TimedUnit)i.next()).tick();
			}
			reporter.Log("Time manager: initiating new activity");
			i=clients.iterator();
			while(i.hasNext()){
				((TimedUnit)i.next()).next();
			}
		}
		msg.tag=TermMsg.IDLE;
		msg.count=0;
	}
	NextHandler.TermMsgSend(msg);
  }

}

