/*
 
Copyright (C) 2004-2005  Stichting Mathematisch Centrum,
                         Amsterdam, The Netherlands
 
*/

import java.io.IOException;
import java.io.InterruptedIOException;

import java.util.*;

import com.testingtech.ttcn.tci.TciModuleParameterIdImpl;
import com.testingtech.ttcn.tci.TciEncoding;
import com.testingtech.ttcn.tri.TestAdapter;
import com.testingtech.ttcn.tri.TriMessageImpl;
import com.testingtech.ttcn.tri.TriParameterImpl;
import com.testingtech.ttcn.tri.TriStatusImpl;

import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.TciCDProvided;
import org.etsi.ttcn.tri.*;

/**
 * TTCN-3 system adapter for mCRL specifications.
 * Note that the adapter waits for a message to arrive before it will
 * start initiating messages.
 * Note that the first possible action is always taken. It would be good
 * to use random choice.
 *
 * @author Stefan Blom
 * @version 1.0
 */
public class mCRLadapter implements Runnable, TimedUnit {

private Stepper stepper;
private Thread t;
private boolean go;
private int current_state[]=new int[2];
private TermHandler Handler;
private Reporter reporter;
private String prefix;
private String name;
private TriCommunicationTE Cte;
private Hashtable PortIdTable=new Hashtable();
private Hashtable ComponentIdTable=new Hashtable();
private boolean simtime=false;

public TriStatus map(String name,TriPortId port,TriComponentId com){
	// Store port and component id for sending messages to the test engine later on.
	PortIdTable.put(name,port);
	ComponentIdTable.put(name,com);
	return new TriStatusImpl();
}

private void log(String msg){
	reporter.Log(prefix+msg);
}

public mCRLadapter(Reporter reporter,TriCommunicationTE Cte,String lpo){
	this.reporter=reporter;
	this.Cte=Cte;
	stepper=new Stepper(lpo,reporter);
	int init[]=stepper.getInit();
	current_state[0]=init[0];
	current_state[1]=init[1];
	name="mCRL adapter("+lpo+")";
	prefix=name+": ";
	t=new Thread(this);
	go=false;
	t.start();
	waitforgo();
	log("online");
}

public synchronized void setgo(){
	go=true;
	notifyAll();
	try {
		wait();
	} catch (InterruptedException e){}
}

public synchronized void waitforgo(){
	while (!go) {
		try {
			wait();
		} catch (InterruptedException e){}
	}
}

public void enableSimulatedTime(TimeManager tm){
	Handler=tm.getTermHandler(name);
	tm.add(this);
	simtime=true;
	Handler.Idle();
}

public synchronized void tick(){
	// Explore transitions, find a tick step and take it.
	stepper.explore(current_state);
	while(stepper.next()){
		String label=stepper.getLabel();
		if (label.equals("tick")) {
			log("tick");
			int next_state[]=stepper.getState();
			current_state[0]=next_state[0];
			current_state[1]=next_state[1];
			while(stepper.next()){};
			return;
		}
	}
	// The non-existence of a tick step is bad.
	reporter.Fatal("time progression is blocked");
}

public synchronized void next(){
	Handler.Activate();
	notifyAll();
}

public synchronized void send_halt(){
	go=false;
	notifyAll();
}

public void halt(){
	send_halt();
	try {
		t.join();
	} catch (InterruptedException irq){}
}

public void run(){
	setgo();
	log("enabling activity");
	while(go) TryAction();
	log("Stepper shutting down...");
	stepper.stop();
	log("Stepper shutdown completed.");
}

private synchronized void TryAction(){
	stepper.explore(current_state);
	while(stepper.next()){
		String label=stepper.getLabel();
		if (label.equals("tau")){
			log("tau");
			int next_state[]=stepper.getState();
			current_state[0]=next_state[0];
			current_state[1]=next_state[1];
			while(stepper.next()){}
			return;
		}
		if (label.startsWith("send")){
			int open=label.indexOf("(");
			int comma=label.indexOf(",");
			int close=label.lastIndexOf(")");
			String port=label.substring(open+1,comma);
			String msg=label.substring(comma+1,close);
			TriPortId PortId=(TriPortId)PortIdTable.get(port);
			TriComponentId ComponentId=(TriComponentId)ComponentIdTable.get(port);
			if (simtime) Handler.Send();
			log(label);
			Cte.triEnqueueMsg(PortId,null,ComponentId,new TriMessageImpl(msg.getBytes()));
			int next_state[]=stepper.getState();
			current_state[0]=next_state[0];
			current_state[1]=next_state[1];
			while(stepper.next()){}
			return;
		}
	}
	try {
		if (simtime) {
			log("idle");
			Handler.Idle();
		}
        	wait();
	} catch (InterruptedException e) {}
}

private synchronized boolean TryAction(String action){
	stepper.explore(current_state);
	while(stepper.next()){
		String label=stepper.getLabel();
		if (label.equals(action)) {
			int next_state[]=stepper.getState();
			current_state[0]=next_state[0];
			current_state[1]=next_state[1];
			while(stepper.next()){}
			if (simtime) Handler.Recv();
			log(label);
			notifyAll();
			return true;
		}
	}
	return false;
}

/* send message from test system to SUT */
public TriStatus send(String port,TriMessage trimsg){
	String msg=new String(trimsg.getEncodedMessage());
	String action="recv("+port+","+msg+")";
	if (TryAction(action)) {
		return new TriStatusImpl();
	} else {
		return new TriStatusImpl("action "+action+" not in menu");
	}
}

}
